# Import classes and functions from RPE_Utils
from AutoRPE.UtilsRPE.MainFunctions import parse_sources, preprocess_sources
from AutoRPE.UtilsRPE.Functions import load_vault, in_which_block
from AutoRPE.UtilsRPE.track_dependecy import track_dependencies, propagate_dependencies
from os import listdir, mkdir
from os.path import join, isdir
from shutil import rmtree
import re
import networkx as nx


if __name__ == "__main__":
    # Getting Command line options
    from optparse import OptionParser

    parser = OptionParser()
    parser.add_option("-v", "--vault", dest="vault", default=None, metavar="FILE")
    parser.add_option("-w", "--working-precision", dest="working_precision", default=23)
    parser.add_option("-o", "--output-vault", dest="output_vault", default="vault.pkl", metavar="FILE")
    parser.add_option("-t", "--omit-tracking", dest="track", default=True, action="store_false")
    parser.add_option("-g", "--export-graph", dest="export_graph", default=False, action="store_true")
    parser.add_option("-s", "--input-source", dest="input_source",
                      default=None,metavar="FILE")

    (options, args) = parser.parse_args()
    vault_path = options.vault
    wp = options.working_precision
    output_vault = options.output_vault
    path_to_input_sources = options.input_source
    export_graph = options.export_graph
    track = options.track

    if path_to_input_sources is None and vault_path is None:
        print("\nOne of the arguments -s/--input-source or -v/--vault is mandatory\n")
        parser.print_help()
        exit(1)

    print("Creating a database from sources found in:\n   %s\n\n" % path_to_input_sources)

    # In case no vault has been provided it will create it from the sources in the input sources folder
    if vault_path is None:
        # Paths to source folders
        # Path_to_input_sources
        path_to_preprocessed_sources = "./tmp_processedsources/"
        path_to_output_sources = "./tmp_outputsources/"
        if not isdir(path_to_preprocessed_sources):
            mkdir(path_to_preprocessed_sources)
        if not isdir(path_to_output_sources):
            mkdir(path_to_output_sources)
        # List of files that should not be processed, as red from "list_of_files_to_keep_unmodified.txt"
        preprocess_blacklist = listdir(path_to_input_sources)

        # Format text + replace real declarations with type(rpe_var)
        preprocess_sources(path_to_input_sources, path_to_preprocessed_sources, blacklist=preprocess_blacklist)

        # Obtain source information
        vault = parse_sources(path_to_preprocessed_sources, save_vault=False)
    else:
        vault = load_vault(vault_path)

    # Lookup table with the significant bits value that correspond to each precision specifier
    lookup_table = {'WP': wp, 'default': wp, '8': 52, 'dp': 52, 'sp': 23, 'fbsp': 23, 'fbdp': 52, 'wp': wp, 'KIND=dp': 52,
                    "KIND=wp": wp,  'kind=fbsp': 23, "kind=sp": 23, "kind=wp": wp, "KIND=8": 52}

    # Searching again through the sources to find the type (single or double) of each variable in the vault

    real_types = ["single", "double"]
    real_variables = [v for v in vault.variables if v.type in real_types]

    length = len(real_variables)

    # Assigning id and precision
    type_to_precision = {"single": 23, "double": 52}
    for index, var in enumerate(real_variables):
        var.id = index
        var.precision = type_to_precision[var.type]

    if track:
        # Track dependencies and propagate it
        # Create empty graph
        graph = nx.DiGraph()
        track_dependencies(path_to_preprocessed_sources, vault, graph=graph)
        if export_graph:
            nx.readwrite.gexf.write_gexf(graph, "./graph.gexf")
        propagate_dependencies(vault, graph=graph)

    # Save the vault with the variable precisions updated.
    # This workaround is needed to allow pickle to write the vault file
    import sys
    import pickle

    sys.setrecursionlimit(20000)
    pickle.dump(vault, open(output_vault, "wb"))

    # Deleting temporary folders
    if isdir(path_to_preprocessed_sources):
        rmtree(path_to_preprocessed_sources)
    if isdir(path_to_output_sources):
        rmtree(path_to_output_sources)
